@@ -72,18 +72,15 @@ struct _GstVorbisEnc {
   gint             channels;
   gint             frequency;
 
+  guint64          samples_in;
   guint64          samples_out;
+  guint64          bytes_out;
 
   GstTagList *     tags;
 
   gboolean         setup;
   gboolean         header_sent;
   gchar           *last_message;
-
-  int              long_size, short_size;
-  int              last_size;
-  int              vorbis_log2_num_modes;
-  int              vorbis_mode_sizes[256];
 };
 
 struct _GstVorbisEncClass {
