@@ -26,10 +26,7 @@
 #include <gst/audio/audio-enumtypes.h>
 #include <gst/audio/audio-format.h>
 #include <gst/audio/audio-channels.h>
-#include <gst/audio/audio-channel-mixer.h>
 #include <gst/audio/audio-info.h>
-#include <gst/audio/audio-quantize.h>
-#include <gst/audio/audio-converter.h>
 
 G_BEGIN_DECLS
 
@@ -54,34 +51,6 @@ G_BEGIN_DECLS
 #define GST_CLOCK_TIME_TO_FRAMES(clocktime, rate) \
   gst_util_uint64_scale_round (clocktime, rate, GST_SECOND)
 
-/* metadata macros */
-
-/**
- * GST_META_TAG_AUDIO_STR:
- *
- * This metadata is relevant for audio streams.
- *
- * Since: 1.2
- */
-#define GST_META_TAG_AUDIO_STR "audio"
-/**
- * GST_META_TAG_AUDIO_CHANNELS_STR:
- *
- * This metadata stays relevant as long as channels are unchanged.
- *
- * Since: 1.2
- */
-#define GST_META_TAG_AUDIO_CHANNELS_STR "channels"
-
-/**
- * GST_META_TAG_AUDIO_RATE_STR:
- *
- * This metadata stays relevant as long as sample rate is unchanged.
- *
- * Since: 1.8
- */
-#define GST_META_TAG_AUDIO_RATE_STR "rate"
-
 /*
  * this library defines and implements some helper functions for audio
  * handling
