@@ -20,8 +20,8 @@
 #ifndef __GST_PB_UTILS_MISSING_PLUGINS_H__
 #define __GST_PB_UTILS_MISSING_PLUGINS_H__
 
-#include <gst/gst.h>
-#include <gst/pbutils/pbutils-prelude.h>
+#include <gst/gstmessage.h>
+#include <gst/gstcaps.h>
 
 G_BEGIN_DECLS
 
@@ -29,23 +29,18 @@ G_BEGIN_DECLS
  * functions to create missing-plugin messages, for use by plugins primarily
  */
 
-GST_PBUTILS_API
 GstMessage * gst_missing_uri_source_message_new (GstElement    * element,
                                                  const gchar   * protocol);
 
-GST_PBUTILS_API
 GstMessage * gst_missing_uri_sink_message_new   (GstElement    * element,
                                                  const gchar   * protocol);
 
-GST_PBUTILS_API
 GstMessage * gst_missing_element_message_new    (GstElement    * element,
                                                  const gchar   * factory_name);
 
-GST_PBUTILS_API
 GstMessage * gst_missing_decoder_message_new    (GstElement    * element,
                                                  const GstCaps * decode_caps);
 
-GST_PBUTILS_API
 GstMessage * gst_missing_encoder_message_new    (GstElement    * element,
                                                  const GstCaps * encode_caps);
 
@@ -53,13 +48,10 @@ GstMessage * gst_missing_encoder_message_new    (GstElement    * element,
  * functions for use by applications when dealing with missing-plugin messages
  */
 
-GST_PBUTILS_API
 gchar       * gst_missing_plugin_message_get_installer_detail (GstMessage * msg);
 
-GST_PBUTILS_API
 gchar       * gst_missing_plugin_message_get_description (GstMessage * msg);
 
-GST_PBUTILS_API
 gboolean      gst_is_missing_plugin_message (GstMessage * msg);
 
 
@@ -69,19 +61,14 @@ gboolean      gst_is_missing_plugin_message (GstMessage * msg);
  * missing-plugin messages posted by elements such as playbin or decodebin
  */
 
-GST_PBUTILS_API
 gchar * gst_missing_uri_source_installer_detail_new (const gchar * protocol);
 
-GST_PBUTILS_API
 gchar * gst_missing_uri_sink_installer_detail_new (const gchar * protocol);
 
-GST_PBUTILS_API
 gchar * gst_missing_element_installer_detail_new (const gchar * factory_name);
 
-GST_PBUTILS_API
 gchar * gst_missing_decoder_installer_detail_new (const GstCaps * decode_caps);
 
-GST_PBUTILS_API
 gchar * gst_missing_encoder_installer_detail_new (const GstCaps * encode_caps);
 
 G_END_DECLS
