@@ -47,7 +47,6 @@ G_BEGIN_DECLS
 #define GST_RIFF_TAG_idx1 GST_MAKE_FOURCC ('i','d','x','1')
 #define GST_RIFF_TAG_dmlh GST_MAKE_FOURCC ('d','m','l','h')
 #define GST_RIFF_TAG_ID32 GST_MAKE_FOURCC ('I','D','3','2')
-#define GST_RIFF_TAG_id3  GST_MAKE_FOURCC ('i','d','3',' ')
 #define GST_RIFF_TAG_IDVX GST_MAKE_FOURCC ('I','D','V','X')
 /* WAV stuff */
 #define GST_RIFF_TAG_fmt  GST_MAKE_FOURCC ('f','m','t',' ')
@@ -60,8 +59,6 @@ G_BEGIN_DECLS
 #define GST_RIFF_TAG_acid GST_MAKE_FOURCC ('a','c','i','d')
 #define GST_RIFF_TAG_labl GST_MAKE_FOURCC ('l','a','b','l')
 #define GST_RIFF_TAG_note GST_MAKE_FOURCC ('n','o','t','e')
-#define GST_RIFF_TAG_smpl GST_MAKE_FOURCC ('s','m','p','l')
-#define GST_RIFF_TAG_inst GST_MAKE_FOURCC ('i','n','s','t')
 
 /* LIST types */
 #define GST_RIFF_LIST_movi GST_MAKE_FOURCC ('m','o','v','i')
@@ -71,7 +68,6 @@ G_BEGIN_DECLS
 #define GST_RIFF_LIST_INFO GST_MAKE_FOURCC ('I','N','F','O')
 #define GST_RIFF_LIST_AVIX GST_MAKE_FOURCC ('A','V','I','X')
 #define GST_RIFF_LIST_adtl GST_MAKE_FOURCC ('a','d','t','l')
-#define GST_RIFF_LIST_ncdt GST_MAKE_FOURCC ('n','c','d','t')
 
 /* fcc types */
 #define GST_RIFF_FCC_vids GST_MAKE_FOURCC ('v','i','d','s')
@@ -113,9 +109,6 @@ G_BEGIN_DECLS
 #define GST_RIFF_INFO_IAAR GST_MAKE_FOURCC ('I','A','A','R') /* album artist */
 #define GST_RIFF_INFO_ITRK GST_MAKE_FOURCC ('I','T','R','K') /* track number */
 
-/* ncdt types - see http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/Nikon.html#NCDT */
-#define GST_RIFF_LIST_nctg GST_MAKE_FOURCC ('n','c','t','g')
-
 /*********Chunk Names***************/
 #define GST_RIFF_FF00 GST_MAKE_FOURCC (0xFF,0xFF,0x00,0x00)
 #define GST_RIFF_00   GST_MAKE_FOURCC ('0', '0',0x00,0x00)
@@ -338,7 +331,6 @@ typedef struct _gst_riff_strf_auds {       /* == WaveHeader (?) */
 #define GST_RIFF_WAVE_FORMAT_ITU_G721_ADPCM (0x0040)
 #define GST_RIFF_WAVE_FORMAT_G728_CELP      (0x0041)
 #define GST_RIFF_WAVE_FORMAT_MSG723         (0x0042)
-#define GST_RIFF_WAVE_FORMAT_ITU_G726_ADPCM (0x0045)
 #define GST_RIFF_WAVE_FORMAT_MPEGL12        (0x0050)
 #define GST_RIFF_WAVE_FORMAT_RT24           (0x0052)
 #define GST_RIFF_WAVE_FORMAT_PAC            (0x0053)
