@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -799,7 +799,6 @@ gst_rtcp_packet_get_rb (GstRTCPPacket * packet, guint nth, guint32 * ssrc,
     guint8 * fractionlost, gint32 * packetslost, guint32 * exthighestseq,
     guint32 * jitter, guint32 * lsr, guint32 * dlsr)
 {
-  guint offset;
   guint8 *data;
   guint32 tmp;
 
@@ -808,31 +807,18 @@ gst_rtcp_packet_get_rb (GstRTCPPacket * packet, guint nth, guint32 * ssrc,
       packet->type == GST_RTCP_TYPE_SR);
   g_return_if_fail (packet->rtcp != NULL);
   g_return_if_fail (packet->rtcp->map.flags & GST_MAP_READ);
-  g_return_if_fail (nth < packet->count);
-
-  /* get offset in 32-bits words into packet, skip the header */
-  if (packet->type == GST_RTCP_TYPE_RR)
-    offset = 2;
-  else
-    offset = 7;
-
-  /* move to requested index */
-  offset += (nth * 6);
-
-  /* check that we don't go past the packet length */
-  if (offset > packet->length)
-    return;
-
-  /* scale to bytes */
-  offset <<= 2;
-  offset += packet->offset;
-
-  /* check if the packet is valid */
-  if (offset + 24 > packet->rtcp->map.size)
-    return;
 
   data = packet->rtcp->map.data;
-  data += offset;
+
+  /* skip header */
+  data += packet->offset + 4;
+  if (packet->type == GST_RTCP_TYPE_RR)
+    data += 4;
+  else
+    data += 24;
+
+  /* move to requested index */
+  data += (nth * 24);
 
   if (ssrc)
     *ssrc = GST_READ_UINT32_BE (data);
@@ -2068,7 +2054,7 @@ gst_rtcp_packet_fb_set_fci_length (GstRTCPPacket * packet, guint16 wordlen)
   wordlen += 2;
   GST_WRITE_UINT16_BE (data, wordlen);
 
-  packet->rtcp->map.size = packet->offset + ((wordlen + 1) * 4);
+  packet->rtcp->map.size += wordlen * 4;
 
   return TRUE;
 }
