@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -267,7 +267,7 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   GstClockTime pts, dts;
   guint16 seqnum;
   guint32 rtptime;
-  gboolean discont, buf_discont;
+  gboolean discont;
   gint gap;
   GstRTPBuffer rtp = { NULL };
 
@@ -281,7 +281,8 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   if (G_UNLIKELY (!gst_rtp_buffer_map (in, GST_MAP_READ, &rtp)))
     goto invalid_buffer;
 
-  buf_discont = GST_BUFFER_IS_DISCONT (in);
+  if (!priv->discont)
+    priv->discont = GST_BUFFER_IS_DISCONT (in);
 
   pts = GST_BUFFER_PTS (in);
   dts = GST_BUFFER_DTS (in);
@@ -297,10 +298,10 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   rtptime = gst_rtp_buffer_get_timestamp (&rtp);
   gst_rtp_buffer_unmap (&rtp);
 
-  discont = buf_discont;
+  discont = FALSE;
 
   GST_LOG_OBJECT (filter, "discont %d, seqnum %u, rtptime %u, pts %"
-      GST_TIME_FORMAT ", dts %" GST_TIME_FORMAT, buf_discont, seqnum, rtptime,
+      GST_TIME_FORMAT ", dts %" GST_TIME_FORMAT, priv->discont, seqnum, rtptime,
       GST_TIME_ARGS (pts), GST_TIME_ARGS (dts));
 
   /* Check seqnum. This is a very simple check that makes sure that the seqnums
@@ -334,15 +335,13 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   }
   priv->next_seqnum = (seqnum + 1) & 0xffff;
 
-  if (G_UNLIKELY (discont)) {
+  if (G_UNLIKELY (discont && !priv->discont)) {
+    GST_LOG_OBJECT (filter, "mark DISCONT on input buffer");
+    /* we detected a seqnum discont but the buffer was not flagged with a discont,
+     * set the discont flag so that the subclass can throw away old data. */
     priv->discont = TRUE;
-    if (!buf_discont) {
-      /* we detected a seqnum discont but the buffer was not flagged with a discont,
-       * set the discont flag so that the subclass can throw away old data. */
-      GST_LOG_OBJECT (filter, "mark DISCONT on input buffer");
-      in = gst_buffer_make_writable (in);
-      GST_BUFFER_FLAG_SET (in, GST_BUFFER_FLAG_DISCONT);
-    }
+    in = gst_buffer_make_writable (in);
+    GST_BUFFER_FLAG_SET (in, GST_BUFFER_FLAG_DISCONT);
   }
 
   bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (filter);
@@ -644,7 +643,7 @@ static gboolean
 gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
     GstEvent * event)
 {
-  GstClockTime timestamp, duration, position;
+  GstClockTime timestamp, duration;
   GstEvent *sevent;
   const GstStructure *s;
 
@@ -657,10 +656,6 @@ gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
   gst_structure_get_clock_time (s, "timestamp", &timestamp);
   gst_structure_get_clock_time (s, "duration", &duration);
 
-  position = timestamp;
-  if (duration != -1)
-    position += duration;
-
   /* send GAP event */
   sevent = gst_event_new_gap (timestamp, duration);
 
