@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 
@@ -23,6 +23,9 @@
 #define __GST_TAG_TAG_H__
 
 #include <gst/gst.h>
+#include <gst/tag/gsttagdemux.h>
+#include <gst/tag/gsttagmux.h>
+#include <gst/tag/xmpwriter.h>
 
 G_BEGIN_DECLS
 
@@ -358,8 +361,8 @@ G_BEGIN_DECLS
  * @GST_TAG_IMAGE_TYPE_BAND_ARTIST_LOGO      : Band/artist logotype
  * @GST_TAG_IMAGE_TYPE_PUBLISHER_STUDIO_LOGO : Publisher/studio logotype
  *
- * Type of image contained in an image tag (specified as field in
- * the image buffer's caps structure)
+ * Type of image contained in an image tag (specified as "image-type" field in
+ * the info structure in the image's #GstSample)
  */
 /* Note: keep in sync with register_tag_image_type_enum() */
 typedef enum {
