@@ -834,10 +834,10 @@ gst_adder_sink_event (GstCollectPads * pads, GstCollectData * pad,
     }
     case GST_EVENT_FLUSH_START:
       /* ensure that we will send a flush stop */
-      GST_COLLECT_PADS_STREAM_LOCK (adder->collect);
-      adder->flush_stop_pending = TRUE;
       res = gst_collect_pads_event_default (pads, pad, event, discard);
       event = NULL;
+      GST_COLLECT_PADS_STREAM_LOCK (adder->collect);
+      adder->flush_stop_pending = TRUE;
       GST_COLLECT_PADS_STREAM_UNLOCK (adder->collect);
       break;
     case GST_EVENT_FLUSH_STOP:
@@ -868,9 +868,20 @@ gst_adder_sink_event (GstCollectPads * pads, GstCollectData * pad,
       adder->pending_events = g_list_append (adder->pending_events, event);
       event = NULL;
       break;
-    case GST_EVENT_SEGMENT:
+    case GST_EVENT_SEGMENT:{
+      const GstSegment *segment;
+      gst_event_parse_segment (event, &segment);
+      if (segment->rate != adder->segment.rate) {
+        GST_ERROR_OBJECT (pad->pad,
+            "Got segment event with wrong rate %lf, expected %lf",
+            segment->rate, adder->segment.rate);
+        res = FALSE;
+        gst_event_unref (event);
+        event = NULL;
+      }
       discard = TRUE;
       break;
+    }
     default:
       break;
   }
